#include <TimerOne.h>

/*******************************************
程序名：蓝牙番茄钟
Arduino：Bluno Beetle V1.0
程序简介：通过蓝牙控制Bluno Beetle启动番茄时间，时间结束后发出声音提醒使用者
休息或者工作，同时附带的继电器可以切断电灯电源。

by dbc0301
2016.5.14

Email:dbc0301@126.com
 *******************************************/
//2,3,4,5分别连接74LS154的A,B,C,D四个引脚
     const int A = 5;
     const int B = 4;
     const int C = 3;
     const int D = 2;
     const int switchLED = A0;//LED显示控制引脚，高电平时有效
     const int speaker = A1;//扬声器引脚
     const int relay = A2;//继电器引脚
     const int frequency = 3400;//闹钟频率
     unsigned long startTime;
     unsigned long endTime;
     int timer;//存储LED发光时应亮的个数
     String comdata = "";
     unsigned int alarmMins;//输入的定时器(单位：分钟)
     int mode = 0;//控制二级菜单
     int loopState = 1;//alarm程序的开关
     int SecondGradeMenu = 0;//二级菜单的开关

/********************************************
74LS154控制函数
函数名：pinLED
函数功能：控制74LS154从0到15一共16个引脚的电平情况
入口参数：number（范围0-15），控制引脚0-15中的一个输出高电平
 ********************************************/
void pinLED(int number) {
     //将十进制数number利用BCD码转换成二进制
     int pinA = number%2;
     int pinB = (number/2)%2;
     int pinC = (number/4)%2;
     int pinD = (number/8)%2;
     digitalWrite(A , pinA);
     digitalWrite(B , pinB);
     digitalWrite(C , pinC);
     digitalWrite(D , pinD);
}

/********************************************
定时器灯控制函数
函数名：timeLED
函数功能：控制从0到i号LED发
入口参数：i（范围0-11），控制从0到i号LED发光
 ********************************************/
void timeLED(int i){
     for(int j=0;j<=i;j++){
     pinLED(j);
     }
}

/********************************************
定时器控制函数
函数名：alarm
函数功能：倒计时minutes分钟后结束函数
入口参数：minutes
 ********************************************/
void alarm(int minutes){
     digitalWrite(switchLED , LOW);//打开LED时间显示
     startTime = millis();
     endTime = startTime;
     while(endTime-startTime<=minutes*60000 & loopState){
          timer = minutes/5-(endTime-startTime)/300000;
          timeLED(timer);
          endTime = millis();
     }
}

/********************************************
提示音控制函数
函数名：speakerAlarm
函数功能：控制扬声器发出哔哔的闹铃声
入口参数：i，控制哔哔声的次数
 ********************************************/
void speakerAlarm(int i){
     for(int j=1;j<=i;j++){
          tone(speaker , frequency);
          delay(100);
          noTone(speaker);
          delay(100);
          tone(speaker , frequency);
          delay(100);
          noTone(speaker);
          delay(650);
     }
}

void setup() {
     Serial.begin(115200);//串口初始化
     pinMode(switchLED , OUTPUT);
     digitalWrite(switchLED , HIGH);//初始化时关闭LED时间显示
     pinMode(A , OUTPUT);
     pinMode(B , OUTPUT);
     pinMode(C , OUTPUT);
     pinMode(D , OUTPUT);
     pinMode(speaker , OUTPUT);
     pinMode(relay , OUTPUT);
     digitalWrite(relay , HIGH);
     Timer1.initialize(1000000);
     Timer1.attachInterrupt( com );
//输出菜单
     Serial.println("1.Start Work.");
     Serial.println("2.Alarm.");
     Serial.println("3.Open the Light.");
     Serial.println("4.Close the Light.");
     Serial.println();
}

void loop() {
     if(Serial.available() > 0){
     switch(Serial.read()){
     case '1'://**********工作(25分钟工作，5分钟休息)**********
         SecondGradeMenu = 1;//打开二级菜单
         Serial.println("Working!");
         Serial.println();
         Serial.println("1.End Work.");
         Serial.println();
         mode = 1;//主菜单1下的二级菜单
         alarm(25);//定时25分钟
         digitalWrite(switchLED , HIGH);//关闭LED时间显示
         if(loopState){
           Serial.println("Time's up");
           Serial.println("Five minutes to rest");
           Serial.println();
           digitalWrite(relay , LOW);
           speakerAlarm(15);
           alarm(5);//定时5分钟
           digitalWrite(switchLED , HIGH);//关闭LED时间显示
           SecondGradeMenu = 0;//关闭二级菜单
           mode = 0;
         if(loopState){
           Serial.println("Time's up");
           Serial.println();
           digitalWrite(relay , LOW);
           speakerAlarm(15);
         }
         }
         loopState = 1;//若中途关闭定时，则在程序结束后重新打开定时器开关
         Serial.println("1.Start Work.");
         Serial.println("2.Alarm.");
         Serial.println("3.Open the Light.");
         Serial.println("4.Close the Light.");
         Serial.println();
         break;
     case '2'://**********定时器**********
         comdata = "";
         Serial.println("Enter minutes.");
         while(!Serial.available()){};
         while(Serial.available() > 0){
              comdata += char(Serial.read());
              delay(3);//为了防止数据丢失,在此设置短暂延时delay(2)
         }
         if (comdata.length() > 0){
         alarmMins = comdata.toInt();//在此把comdata转化成INT型数值,以备后续使用
         SecondGradeMenu = 1;//打开二级菜单
         mode = 1;//主菜单1下的二级菜单
         Serial.print("Countdown ");
         Serial.print(alarmMins);
         Serial.println(" minutes");
         alarm(alarmMins);
         Serial.println();
         Serial.println("1.End Work.");
         Serial.println();
         digitalWrite(switchLED , HIGH);//关闭LED时间显示
         SecondGradeMenu = 0;//关闭二级菜单
         mode = 0;
         if(loopState){
           Serial.println("Time's up");
           speakerAlarm(15);
         }
         loopState = 1;//若中途关闭定时，则在程序结束后重新打开定时器开关
         }
         comdata = "";//  必须在此把comdata设为空字符,否则会导致前后字符串叠加
         break;
     case '3':
         digitalWrite(relay , HIGH);
         break;
     case '4':
         digitalWrite(relay , LOW);    
     default:
         Serial.println("1.Start Work.");
         Serial.println("2.Alarm.");
         Serial.println("3.Open the Light.");
         Serial.println("4.Close the Light.");
         Serial.println();
         break;
     }
     }
}

/********************************************
串口读取控制函数
函数名：com
函数功能：由timer1库每1s中断一，读取串口后操作二级菜单
入口参数：无
 ********************************************/

void com(){
  if(SecondGradeMenu){
     switch(mode){
     case 1:
         switch(Serial.read()){
         case '1':
             loopState = 0;//关闭定时器
             Serial.println("Alarm Off.");
             Serial.println();
             break;
         default:
             break;
         }
     default:
         break;
     }
  }
}
